/********************************************************************************************
 *                                                                                          *
 * Copyright (C) 2017 Armin Felder, Dennis Beier                                            *
 * This file is part of RocketChatMobileEngine <https://git.fairkom.net/chat/fairchat>.     *
 *                                                                                          *
 * RocketChatMobileEngine is free software: you can redistribute it and/or modify           *
 * it under the terms of the GNU General Public License as published by                     *
 * the Free Software Foundation, either version 3 of the License, or                        *
 * (at your option) any later version.                                                      *
 *                                                                                          *
 * RocketChatMobileEngine is distributed in the hope that it will be useful,                *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                            *
 * GNU General Public License for more details.                                             *
 *                                                                                          *
 * You should have received a copy of the GNU General Public License                        *
 * along with RocketChatMobileEngine. If not, see <http://www.gnu.org/licenses/>.           *
 *                                                                                          *
 ********************************************************************************************/



#ifndef USERMODEL_H
#define USERMODEL_H
#include <QAbstractListModel>
#include <QSharedPointer>
#include <QMultiMap>
#include <QSet>
#include <QDebug>
#include <QMap>
#include <QList>

#include "repos/entities/rocketchatuser.h"
class UserModel: public QAbstractListModel
{
        Q_OBJECT
        Q_PROPERTY( QString currentChannel READ getCurrent WRITE setCurrent NOTIFY currentChannelChanged )
        enum UserRoles {
            UserName = Qt::UserRole + 1,
            UserId
        };
    public:
        UserModel( QObject *parent = 0 );
        int rowCount( const QModelIndex &parent = QModelIndex() ) const;
        QVariant data( const QModelIndex &index, int role = Qt::DisplayRole ) const;
        bool insertUser( const QString &, const QSharedPointer<RocketChatUser> & );
        QString getCurrent() const;
        void setCurrent( const QString &value );
        void onCurrentChannelChanged( const QString &newText );

    protected:
        QHash<int, QByteArray> roleNames() const;
        QSet<QString> duplicateCheck;
        QMultiMap<QString, QSharedPointer<RocketChatUser>> userMap;
        QMap<QString,QSharedPointer<RocketChatUser>> userOfCurrentChannel;
        QString current;
    signals:
        void currentChannelChanged( const QString &newText );
        void countChanged();
};

#endif // USERMODEL_H
