/********************************************************************************************
 *                                                                                          *
 * Copyright (C) 2017 Armin Felder, Dennis Beier                                            *
 * This file is part of RocketChatMobileEngine <https://git.fairkom.net/chat/fairchat>.     *
 *                                                                                          *
 * RocketChatMobileEngine is free software: you can redistribute it and/or modify           *
 * it under the terms of the GNU General Public License as published by                     *
 * the Free Software Foundation, either version 3 of the License, or                        *
 * (at your option) any later version.                                                      *
 *                                                                                          *
 * RocketChatMobileEngine is distributed in the hope that it will be useful,                *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                            *
 * GNU General Public License for more details.                                             *
 *                                                                                          *
 * You should have received a copy of the GNU General Public License                        *
 * along with RocketChatMobileEngine. If not, see <http://www.gnu.org/licenses/>.           *
 *                                                                                          *
 ********************************************************************************************/


#ifndef ROCKETCHATCHANNEL_H
#define ROCKETCHATCHANNEL_H

#include <QObject>
#include <QJsonObject>
#include <QString>
#include <QMap>
#include <QList>
#include <QJsonDocument>
#include <QDebug>
#include <QJsonArray>
#include <QLinkedList>
#include <QtConcurrent>

#include "repos/messagerepository.h"
#include "repos/channelrepository.h"
#include "repos/entities/rocketchatuser.h"
#include "repos/entities/rocketchatmessage.h"

#include "api/restapi.h"
#include "api/meteorddp.h"
#include "restRequests/postrequest.h"
#include "restRequests/restsendmessagerequest.h"
#include "ddpRequests/ddpmethodrequest.h"
#include "ddpRequests/ddpsubscriptionrequest.h"
#include "ddpRequests/rocketchatsubscribechannelrequest.h"

#include "utils.h"
#include "persistancelayer.h"
#include "rocketchatserver.h"
#include "services/messageservice.h"

typedef QSharedPointer<RestRequest> RestApiRequest;

class PersistanceLayer;
class RocketChatServerData;

class RocketChatChannel : public QObject
{
        Q_OBJECT
    public:
        RocketChatChannel( RocketChatServerData *pServer, MessageService *pMessageService, QString pRoomId, QString pName, QString pType );

        ~RocketChatChannel();

        bool addMessage( const ChatMessage, bool nosiginal = false );
        void setRoomId( const QString &pValue );
        void setName( const QString &pValue );
        void setUsers( const QMap<QString, QSharedPointer<RocketChatUser> > &pValue );
        void setUsers( const QList<QSharedPointer<RocketChatUser>> &pValue );
        void setJoined( bool pValue );
        void setUnreadMessages( unsigned int pValue );
        bool addUser( QSharedPointer<RocketChatUser> pUser );
        QList<QSharedPointer<RocketChatMessage>> addMessages( QList<QSharedPointer<RocketChatMessage>> pMessagesList );

        QHash<QString, QSharedPointer< RocketChatMessage >> getMessages() const;
        QString getRoomId() const;
        QString getName() const;
        QMap<QString, QSharedPointer<RocketChatUser> > getUsers() const;
        ChatMessage getYoungestMessage();
        ChatMessage getOldestMessage();
        MessageRepository *getMessageRepo( void );
        bool getJoined() const;
        unsigned int getUnreadMessages() const;
        QString getType() const;

        bool getReadOnly() const;
        void setReadOnly( bool pValue );

        QStringList getMuted() const;
        void setMuted( const QStringList &pValue );

        bool getArchived() const;
        void setArchived( bool pValue );

        QJsonObject toJsonObject( void );

        void sendUnsentMessages( void );

        bool getSelfMuted() const;
        void setSelfMuted( bool pValue );

        void setParentRepo( ChannelRepository *pParentRepo );
        int operator >( RocketChatChannel &channel );

        bool getBlocked() const;
        void setBlocked( bool blocked );

        bool getDeleted() const;
        void setDeleted( bool deleted );

    protected:
        bool mDeleted;
        bool mReadOnly = false;
        bool mArchived = false;
        bool mBlocked = false;
        QStringList mMuted;
        QString mRoomId;
        QString mName;
        MessageRepository mMessages;
        QMap<QString, QSharedPointer<RocketChatUser>> mUsers;
        bool mJoined = false;
        QString mType;
        unsigned int mUnreadMessages = 0;
        bool mSelfMuted = false;
    signals:
        void messageListReceived( QString pChannelId, QList<ChatMessage> pMessages );
        void messageAdded( QString id, qint64 timestamp );
        void unreadMessagesChanged( QString id, int number );
        void channelDeleted( const QString &pId, bool deleted );
        void dataChanged( QString id, QString property );

};

#endif // ROCKETCHATCHANNEL_H
