#include "websocketwinrt.h"

#include <QDebug>
#include <QString>
#include <QUrl>
#include <QObject>
#include <QThread>
#include <QTimer>

#include <vccorlib.h>
#include <collection.h>
#include <ppltasks.h>
#include <windows.networking.h>
#include <windows.networking.sockets.h>
#include <windows.storage.streams.h>
#include <windows.foundation.h>
#include <windows.ui.core.h>

#include <iostream>

using namespace Windows::Networking::Sockets;
using namespace Windows::Storage::Streams;
using namespace Windows::Foundation;
using namespace Platform;
using namespace Windows::Web;
using namespace Windows::UI::Core;
using namespace concurrency;
/*
HRESULT __stdcall GetActivationFactoryByPCWSTR(void*,Guid&, void**);
namespace __winRT
{
    HRESULT __stdcall __getActivationFactoryByPCWSTR(const void* str, ::Platform::Guid& pGuid, void** ppActivationFactory)
    {
        return GetActivationFactoryByPCWSTR(const_cast<void*>(str), pGuid, ppActivationFactory);
    }
}*/

WinRTQTSignalHelperClass* WinRTQTSignalHelperClass::helper = NULL;

WinRTQTSignalHelperClass* WinRTQTSignalHelperClass::instance(){
    if(!helper){
        helper = new WinRTQTSignalHelperClass;
    }
    return helper;
}



WinRTWebsocket::WinRTWebsocket()
{
    helper = WinRTQTSignalHelperClass::instance();
    connect(helper,&WinRTQTSignalHelperClass::onMessageReceived,this,&WinRTWebsocket::textMessageReceived, Qt::UniqueConnection );
    connect(helper,&WinRTQTSignalHelperClass::onClosed,this,&WinRTWebsocket::closed, Qt::UniqueConnection );
    connect(helper, &WinRTQTSignalHelperClass::onConnected, this, &WinRTWebsocket::connected, Qt::UniqueConnection );
}


void WinRTWebsocket::open(QUrl url){
    QString urlString = url.toString();
    std::wstring urlWString = urlString.toStdWString();
    Platform::String^ refUrl = ref new Platform::String(urlWString.c_str());
    Uri^ server = ref new Uri(refUrl);
    serverAdrr = server;
    m_context = ref new ReceiveContext;
    m_context->init(server);

}

void WinRTWebsocket::checkForConnection(){
    if(connectionOpen){
        timer->stop();
        disconnect(timer,&QTimer::timeout,this,&WinRTWebsocket::checkForConnection);
        emit(connected());
    }
    else{
        //init(serverAdrr);
    }
}

void WinRTWebsocket::close(){
    m_context->close();
}

void WinRTWebsocket::sendTextMessage(QString message){

    m_context->SendMessage(WinRTWebsocket::QStringToPlatformString(message));

}

bool WinRTWebsocket::isValid()
{
    //todo: fix it!
    return true;
}

void WinRTWebsocket::emitOnMessageReceived(QString message)
{
    //emit(MessageReceived(message));
}

void WinRTWebsocket::emitOnClosed(){
	close();
    emit(closed());
}

void WinRTQTSignalHelperClass::emitOnMessageReceived(QString message)
{
    emit(onMessageReceived(message));
}

void WinRTQTSignalHelperClass::emitOnClosed(){
    emit(onClosed());
}

void WinRTQTSignalHelperClass::emitOnConnected() {
	emit(onConnected());
}

WinRTWebsocket::~WinRTWebsocket(){
    close();
	disconnect(helper, &WinRTQTSignalHelperClass::onMessageReceived, this, &WinRTWebsocket::textMessageReceived);
	disconnect(helper, &WinRTQTSignalHelperClass::onClosed, this, &WinRTWebsocket::closed);
	disconnect(helper, &WinRTQTSignalHelperClass::onConnected, this, &WinRTWebsocket::connected);
   // delete helper;
    timer->deleteLater();
}


ReceiveContext::ReceiveContext(){
    helper = WinRTQTSignalHelperClass::instance();
}

void ReceiveContext::init(Uri^ server){
    qDebug()<<"WinRT init connection";

    webSocket = ref new MessageWebSocket();

    webSocket->Control->MessageType = SocketMessageType::Utf8;
    webSocket->MessageReceived += ref new TypedEventHandler<MessageWebSocket^, MessageWebSocketMessageReceivedEventArgs^>(this, &ReceiveContext::OnReceive);
    webSocket->Closed += ref new TypedEventHandler<IWebSocket^, WebSocketClosedEventArgs^>(this, &ReceiveContext::Closed);
    qDebug()<<"start connection task";
    webSocket->ConnectAsync(server);
    qDebug()<<"started connection task";
}

void ReceiveContext::SendMessage(Platform::String ^message){

    if(messageWriter){
        messageWriter->WriteString(message);

        messageWriter->StoreAsync();
    }
}

void ReceiveContext::HandleException(Exception^ exception){
    qDebug()<<"exception";
    WebErrorStatus status = WebSocketError::GetStatus(exception->HResult);
    Platform::String^ error = status.ToString();
    qDebug()<<QString::fromStdWString(error->Data());
}

void ReceiveContext::OnReceive(MessageWebSocket^ sender, MessageWebSocketMessageReceivedEventArgs^ args){
	if (!connected) {
		connected = true;
		helper->emitOnConnected();
	}
    if(messageWriter == nullptr){
        messageWebSocket = webSocket;
        messageWriter = ref new DataWriter(webSocket->OutputStream);
    }

		DataReader^ reader = args->GetDataReader();
		reader->UnicodeEncoding = UnicodeEncoding::Utf8;
		String^ read = reader->ReadString(reader->UnconsumedBufferLength);
		helper->emitOnMessageReceived(WinRTWebsocket::PlatformStringToQString(read));
}

void ReceiveContext::Closed(IWebSocket^ sender, WebSocketClosedEventArgs^ args){
    qDebug()<<"closing";

    helper->emitOnClosed();
}

void setupWriter()
{

}

void ReceiveContext::checkMessage(Platform::String^ message)
{
//    if(MessageWebSocket == nullptr){
//        messageWebSocket
//    }
}

void ReceiveContext::close()
{
    try
        {
            if (messageWebSocket != nullptr)
            {
                qDebug()<<"closing connection";
                messageWebSocket->Close(1000, "Closed due to user request.");
                messageWebSocket = nullptr;
            }
            else
            {
                qDebug()<<"connection allready closed";
            }
        }
        catch (Exception^ exception)
        {
            HandleException(exception);
        }
}
